# import os
# import sys

# def add_path(path):
#     if path not in sys.path:
#         sys.path.insert(0, path)

# this_dir = os.path.dirname(__file__)
# # print('Module    sys.path  ==> ', sys.path)
# add_path(this_dir)

from manager import RedisManager